/'****************************************************************************
*
* Name: dataobj.bi
*
* Synopsis: Data object used in various data structures.
*
* Description: The data object is a variant data object that can hold one of
*              the standard FreeBasic data types. 
*              
*
* Copyright 2009, Richard D. Clark
*
*                          The Wide Open License (WOL)
*
* Permission to use, copy, modify, distribute and sell this software and its
* documentation for any purpose is hereby granted without fee, provided that
* the above copyright notice and this license appear in all source copies. 
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF
* ANY KIND. See http://www.dspguru.com/wol.htm for more information.
*
*****************************************************************************'/
'Create a NULL value.
#Ifndef NULL
    #Define NULL 0
#EndIf

'Enumeration of allowable types in each data object.
Enum cdatatype
    tNull
    tByte
    tUByte
    tShort
    tUShort
    tInteger
    tUInteger
    tLong
    tUlong
    tLongint
    tUlongint
    tSingle
    tDouble
    tString
End Enum

'Data object type definition.
Type dataobj
    Private:
    _ctype As cdatatype 'Current type of data, default is null.
    Union
        _clbyte As Byte     'Depending on the value of ctype, _cl* will contain the data.
        _clubyte As UByte    'If ctype is tNull, the object will not contain any valid data.
        _clshort As Short
        _clushort As UShort
        _clinteger As Integer
        _cluinteger As UInteger
        _cllong As Long
        _clulong As ULong
        _cllongint As LongInt
        _clulongint As ULongInt
        _clsingle As Single
        _cldouble As Double
        _clstring As ZString Ptr
    End Union
    Declare Sub _ClearData () 
    Public:
    Declare Constructor ()
    Declare Destructor ()
    Declare Property DataType () As cdatatype 'Returns the object data type.
    Declare Sub ClearData ()
    Declare Operator Let (rhs As dataobj) 'Sets data using a dataobj object.
    Declare Operator Let (rhs As Byte)    'Sets object data using variable.
    Declare Operator Let (rhs As UByte) 
    Declare Operator Let (rhs As Short) 
    Declare Operator Let (rhs As UShort) 
    Declare Operator Let (rhs As Integer) 
    Declare Operator Let (rhs As UInteger) 
    Declare Operator Let (rhs As Long) 
    Declare Operator Let (rhs As ULong) 
    Declare Operator Let (rhs As LongInt)
    Declare Operator Let (rhs As ULongInt)
    Declare Operator Let (rhs As Single)
    Declare Operator Let (rhs As Double)
    Declare Operator Let (rhs As String)
    Declare Operator Cast () As Byte     'Cast to a particular data type.
    Declare Operator Cast () As UByte
    Declare Operator Cast () As Short
    Declare Operator Cast () As UShort
    Declare Operator Cast () As Integer
    Declare Operator Cast () As UInteger
    Declare Operator Cast () As Long
    Declare Operator Cast () As ULong
    Declare Operator Cast () As LongInt
    Declare Operator Cast () As ULongInt
    Declare Operator Cast () As Single
    Declare Operator Cast () As Double
    Declare Operator Cast () As String
End Type

'Clears any data in object and sets to Null.
Sub dataobj._ClearData()

    'Based on what the type is, clear the data.
    Select Case _ctype
        Case tNull
            'Nothing to do here.
        Case tByte
            _clbyte = 0
        Case tUByte
            _clubyte = 0
        Case tShort
            _clshort = 0
        Case tUShort
            _clushort = 0
        Case tInteger
            _clinteger = 0
        Case tUInteger
            _cluinteger = 0
        Case tLong
            _cllong = 0
        Case tUlong
            _clulong = 0
        Case tLongint
            _cllongint = 0
        Case tUlongint
            _cllongint = 0
        Case tSingle
            _clsingle = 0
        Case tDouble
            _cldouble = 0
        Case tString
            'Deallocate string if present and set ptr to null.
            If _clstring <> NULL Then
                DeAllocate _clstring
                _clstring = NULL
            EndIf
    End Select
    'Set data type to Null.
    _ctype = tNull
End Sub

'Object constructor sets object to Null.
Constructor dataobj ()
    _ctype = tNull
End Constructor

'Calls _ClearData to clear contents.
Destructor dataobj ()
    _ClearData
End Destructor

'Calls _ClearData to clear contents.
Sub dataobj.ClearData ()
    _ClearData
End Sub

'Returns the object data type.
Property dataobj.DataType () As cdatatype
    Return _ctype
End Property

'Set the value of the data object using another data object.
Operator dataobj.Let (rhs As dataobj)
    
   'Clears any current data and set object to NULL.
   _ClearData
   'Based on what the passed type is, set the data.
   _ctype = rhs.DataType
   Select Case _ctype
   	Case tNull
        'Nothing to do here.
   	Case tByte
         _clbyte = rhs
   	Case tUByte
      	_clubyte = rhs
      Case tShort
         _clshort = rhs
      Case tUShort
         _clushort = rhs
      Case tInteger
         _clinteger = rhs
      Case tUInteger
         _cluinteger = rhs
      Case tLong
         _cllong = rhs
      Case tUlong
         _clulong = rhs
      Case tLongint
         _cllongint = rhs
      Case tUlongint
         _clulongint = rhs
      Case tSingle
         _clsingle = rhs
      Case tDouble
         _cldouble = rhs
    	Case tString 'If a string is passed, allocate space for new string, else set to NULL.
         Dim As String s = rhs
         Dim As Integer l = Len(s)
         If l > 0 Then
            _clstring = Allocate(l + 1)
            *_clstring = s
         Else
            _clstring = NULL
         EndIf
   End Select
    
End Operator

'Sets object data using variable.
Operator dataobj.Let (rhs As Byte) 
    
   'Clear any existing data.
   _ClearData
   _ctype = tByte
   _clbyte = rhs
    
End Operator

Operator dataobj.Let (rhs As UByte)
    
   'Clear any existing data.
   _ClearData
   _ctype = tUByte
   _clubyte = rhs

End Operator

Operator dataobj.Let (rhs As Short)
    
   'Clear any existing data.
   _ClearData
   _ctype = tShort
   _clshort = rhs

End Operator

Operator dataobj.Let (rhs As UShort)
    
   'Clear any existing data.
   _ClearData
   _ctype = tUShort
   _clushort = rhs

End Operator

Operator dataobj.Let (rhs As Integer)
    
   'Clear any existing data.
   _ClearData
   _ctype = tInteger
   _clinteger = rhs

End Operator

Operator dataobj.Let (rhs As UInteger)
    
   'Clear any existing data.
   _ClearData
   _ctype = tUinteger  
   _cluinteger = rhs
End Operator

Operator dataobj.Let (rhs As Long)
    
   'Clear any existing data.
   _ClearData
   _ctype = tLong
   _cllong = rhs

End Operator

Operator dataobj.Let (rhs As ULong)
    
   'Clear any existing data.
   _ClearData
   _ctype = tULong
   _clulong = rhs

End Operator

Operator dataobj.Let (rhs As LongInt)
    
   'Clear any existing data.
   _ClearData
   _ctype = tLongint
   _cllongint = rhs

End Operator

Operator dataobj.Let (rhs As ULongInt)
    
   'Clear any existing data.
   _ClearData
   _ctype = tULongint
   _clulongint = rhs

End Operator

Operator dataobj.Let (rhs As Single)
    
   'Clear any existing data.
   _ClearData
   _ctype = tSingle
   _clsingle = rhs

End Operator

Operator dataobj.Let (rhs As Double)
    
   'Clear any existing data.
   _ClearData
   _ctype = tDouble
   _cldouble = rhs
    
End Operator

Operator dataobj.Let (rhs As String)
   Dim As Integer l = Len(rhs)
    
    'Clear any existing data.
   _ClearData
    'Akllocate spaxe for string, or NULL for empty string.
   _ctype = tString
   If l > 0 Then
      _clstring = Allocate(l + 1)
      *_clstring = rhs
   Else
      _clstring = NULL
   EndIf
    
End Operator

'Cast to data type. Allows returning data into a simple variable of type.
Operator dataobj.Cast () As Byte
   If _ctype = tByte Then
      Return _clbyte
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As UByte
   If _ctype = tuByte Then
       Return _clubyte
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As Short
   If _ctype = tShort Then
       Return _clshort
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As UShort
   If _ctype = tUShort Then
       Return _clushort
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As Integer
   If _ctype = tInteger Then
       Return _clinteger
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As UInteger
   If _ctype = tUInteger Then
       Return _cluinteger
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As Long
   If _ctype = tLong Then
       Return _cllong
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As ULong
   If _ctype = tULong Then
       Return _clulong
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As LongInt
   If _ctype = tLongInt Then
       Return _cllongint
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As ULongInt
   If _ctype = tUlongInt Then
       Return _clulongint
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As Single
   If _ctype = tSingle Then
       Return _clsingle
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As Double
   If _ctype = tDouble Then
       Return _cldouble
   Else
   	Return 0
   EndIf
End Operator

Operator dataobj.Cast () As String
   If _ctype = tString Then
      If _clstring <> NULL Then
         Return *_clstring
      Else
         Return ""
      EndIf
   Else
   	Return ""
   EndIf
End Operator
